@@ -127,18 +127,18 @@ void update_display(lx_battery *lx_b, gboolean repaint) {
     cr = cairo_create(lx_b->pixmap);
     cairo_set_line_width (cr, 1.0);
 
-    /* no battery is found */
-    if( b == NULL ) 
-    {
-	gtk_widget_set_tooltip_text( lx_b->drawingArea, _("No batteries found") );
-	return;
-    }
-    
     /* draw background */
     gdk_cairo_set_source_color(cr, &lx_b->background);
     cairo_rectangle(cr, 0, 0, lx_b->width, lx_b->height);
     cairo_fill(cr);
 
+    /* no battery is found */
+    if( b == NULL ) 
+    {
+	gtk_widget_set_tooltip_text( lx_b->drawingArea, _("No batteries found") );
+	goto update_done;
+    }
+    
     /* fixme: only one battery supported */
 
     rate = lx_b->b->current_now;
@@ -236,6 +236,8 @@ void update_display(lx_battery *lx_b, gboolean repaint) {
         cairo_fill(cr);
 
     }
+
+update_done:
     if( repaint )
 	gtk_widget_queue_draw( lx_b->drawingArea );
 
@@ -245,12 +247,19 @@ void update_display(lx_battery *lx_b, gboolean repaint) {
 
 /* This callback is called every 3 seconds */
 static int update_timout(lx_battery *lx_b) {
+    battery *bat;
     GDK_THREADS_ENTER();
     lx_b->state_elapsed_time++;
     lx_b->info_elapsed_time++;
 
-    /* check the  batteries every 3 seconds */
-    battery_update( lx_b->b );
+    bat = battery_update( lx_b->b );
+    if (bat == NULL)
+    {
+	battery_free(lx_b->b);
+
+	/* maybe in the mean time a battery has been inserted. */
+	lx_b->b = battery_get();
+    }
 
     update_display( lx_b, TRUE );
 
@@ -337,10 +346,6 @@ constructor(Plugin *p, char **fp)
     /* get available battery */
     lx_b->b = battery_get ();
     
-    /* no battery available */
-    if ( lx_b->b == NULL )
-	goto error;
-    
     p->pwid = gtk_event_box_new();
     GTK_WIDGET_SET_FLAGS( p->pwid, GTK_NO_WINDOW );
     gtk_container_set_border_width( GTK_CONTAINER(p->pwid), 1 );
@@ -472,6 +477,9 @@ destructor(Plugin *p)
 
     lx_battery *b = (lx_battery *) p->priv;
 
+    if (b->b != NULL)
+	battery_free(b->b);
+
     if (b->pixmap)
         cairo_surface_destroy(b->pixmap);
 
