@@ -382,7 +382,7 @@ static void dirmenu_apply_configuration_to_children(GtkWidget * w, DirMenuPlugin
         if (dm->name == NULL)
 	    gtk_label_set_text(GTK_LABEL(w), NULL);
         else
-            panel_draw_label_text(dm->plugin->panel, w, dm->name, FALSE, TRUE);
+            panel_draw_label_text(dm->plugin->panel, w, dm->name, FALSE, 1, TRUE);
     }
 }
 
