@@ -17,13 +17,14 @@
  */
 
 #include <string.h>
+#include <stdlib.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include "netstat.h"
 #include "statusicon.h"
 
 struct statusicon *create_statusicon(GtkWidget *box, const char *filename,
-        const char *tooltips)
+        const char *tooltips, const char* icon_name)
 {
     struct statusicon *newicon;
     newicon = malloc(sizeof(struct statusicon));
@@ -37,7 +38,20 @@ struct statusicon *create_statusicon(GtkWidget *box, const char *filename,
     gtk_box_pack_start(GTK_BOX(box), newicon->main, TRUE, TRUE, 0);
 
     /* icon */
-    newicon->icon = gtk_image_new_from_file(filename);
+    
+    /*icon theme*/
+    GtkIconTheme* icon_theme = gtk_icon_theme_get_default();
+    
+    /*if we've got a theme*/
+    if(gtk_icon_theme_has_icon(icon_theme, icon_name)) {
+		GdkPixbuf* temp_pixbuf = gtk_icon_theme_load_icon(icon_theme, icon_name, 24, 0, NULL);
+		newicon->icon = gtk_image_new_from_pixbuf(temp_pixbuf);
+	}
+    
+    /* oh well...*/
+    else {
+		newicon->icon = gtk_image_new_from_file(filename);
+	}
     gtk_container_add(GTK_CONTAINER(newicon->main), newicon->icon);
     gtk_widget_show_all(newicon->main);
 
@@ -68,7 +82,7 @@ void statusicon_destroy(struct statusicon *icon)
 
 void set_statusicon_image_from_file(struct statusicon *widget, const char *filename)
 {
-    gtk_image_set_from_file(widget->icon, filename);
+    gtk_image_set_from_file(GTK_IMAGE(widget->icon), filename);
 }
 
 void set_statusicon_tooltips(struct statusicon *widget, const char *tooltips)
