@@ -120,9 +120,28 @@ static TrayClient * client_lookup(TrayPlugin * tr, Window window)
     return NULL;
 }
 
+static void client_print(TrayPlugin * tr, char c, TrayClient * tc, XClientMessageEvent * xevent)
+{
+    int const log_at_level = LOG_ALL;
+    if (log_at_level <= log_level) {
+        char *name = get_utf8_property(tc->window, a_NET_WM_NAME);
+        int pid = get_net_wm_pid(tc->window);
+        XClientMessageEvent xcm = {0};
+        if (!xevent)
+            xevent = &xcm;
+        LOG(log_at_level, "tray: %c%p, winid 0x%lx: %s (PID %d), plug %p, serial no %lu, send_event %c, format %d\n",
+                c, tc, tc->window, name, pid,
+                gtk_socket_get_plug_window(GTK_SOCKET(tc->socket)),
+                xevent->serial, xevent->send_event ? 'y' : 'n', xevent->format);
+        g_free(name);
+    }
+}
+
 /* Delete a client. */
 static void client_delete(TrayPlugin * tr, TrayClient * tc, gboolean unlink)
 {
+    client_print(tr, '-', tc, NULL);
+
     if (unlink)
     {
         if (tr->client_list == tc)
@@ -455,6 +474,14 @@ static void trayclient_request_dock(TrayPlugin * tr, XClientMessageEvent * xeven
 
     /* Connect the socket to the plug.  This can only be done after the socket is realized. */
     gtk_socket_add_id(GTK_SOCKET(tc->socket), tc->window);
+
+    //fprintf(stderr, "Notice: checking plug %ud\n", tc->window );
+    /* Checks if the plug has been created inside of the socket. */
+    if (gtk_socket_get_plug_window ( GTK_SOCKET(tc->socket) ) == NULL) {
+        //fprintf(stderr, "Notice: removing plug %ud\n", tc->window );
+        icon_grid_remove(tr->icon_grid, tc->socket);
+        return;
+    }
 }
 
 /* GDK event filter. */
@@ -528,9 +555,17 @@ static void tray_unmanage_selection(TrayPlugin * tr)
     {
         GtkWidget * invisible = tr->invisible;
         GdkDisplay * display = gtk_widget_get_display(invisible);
+#if GTK_CHECK_VERSION(2,14,0)
+        if (gdk_selection_owner_get_for_display(display, tr->selection_atom) == gtk_widget_get_window(invisible))
+#else
         if (gdk_selection_owner_get_for_display(display, tr->selection_atom) == invisible->window)
+#endif
         {
+#if GTK_CHECK_VERSION(2,14,0)
+            guint32 timestamp = gdk_x11_get_server_time(gtk_widget_get_window(invisible));
+#else
             guint32 timestamp = gdk_x11_get_server_time(invisible->window);
+#endif
             gdk_selection_owner_set_for_display(
                 display,
                 NULL,
@@ -591,10 +626,18 @@ static int tray_constructor(Plugin * p, char ** fp)
     gtk_widget_add_events(invisible, GDK_PROPERTY_CHANGE_MASK | GDK_STRUCTURE_MASK);
 
     /* Try to claim the _NET_SYSTEM_TRAY_Sn selection. */
+#if GTK_CHECK_VERSION(2,14,0)
+    guint32 timestamp = gdk_x11_get_server_time(gtk_widget_get_window(invisible));
+#else
     guint32 timestamp = gdk_x11_get_server_time(invisible->window);
+#endif
     if (gdk_selection_owner_set_for_display(
         display,
+#if GTK_CHECK_VERSION(2,14,0)
+        gtk_widget_get_window(invisible),
+#else
         invisible->window,
+#endif
         tr->selection_atom,
         timestamp,
         TRUE))
@@ -607,7 +650,11 @@ static int tray_constructor(Plugin * p, char ** fp)
         xev.format = 32;
         xev.data.l[0] = timestamp;
         xev.data.l[1] = selection_atom;
+#if GTK_CHECK_VERSION(2,14,0)
+        xev.data.l[2] = GDK_WINDOW_XWINDOW(gtk_widget_get_window(invisible));
+#else
         xev.data.l[2] = GDK_WINDOW_XWINDOW(invisible->window);
+#endif
         xev.data.l[3] = 0;    /* manager specific data */
         xev.data.l[4] = 0;    /* manager specific data */
         XSendEvent(GDK_DISPLAY_XDISPLAY(display), RootWindowOfScreen(xscreen), False, StructureNotifyMask, (XEvent *) &xev);
@@ -617,7 +664,11 @@ static int tray_constructor(Plugin * p, char ** fp)
         gulong data = SYSTEM_TRAY_ORIENTATION_HORZ;
         XChangeProperty(
             GDK_DISPLAY_XDISPLAY(display),
+#if GTK_CHECK_VERSION(2,14,0)
+            GDK_WINDOW_XWINDOW(gtk_widget_get_window(invisible)),
+#else
             GDK_WINDOW_XWINDOW(invisible->window),
+#endif
             a_NET_SYSTEM_TRAY_ORIENTATION,
             XA_CARDINAL, 32,
             PropModeReplace,
@@ -628,7 +679,11 @@ static int tray_constructor(Plugin * p, char ** fp)
 
         /* Reference the window since it is never added to a container. */
         tr->invisible = invisible;
+#if GTK_CHECK_VERSION(2,14,0)
+        tr->invisible_window = GDK_WINDOW_XWINDOW(gtk_widget_get_window(invisible));
+#else
         tr->invisible_window = GDK_WINDOW_XWINDOW(invisible->window);
+#endif
         g_object_ref(G_OBJECT(invisible));
     }
     else
@@ -640,7 +695,11 @@ static int tray_constructor(Plugin * p, char ** fp)
         
     /* Allocate top level widget and set into Plugin widget pointer. */
     p->pwid = gtk_event_box_new();
+#if GTK_CHECK_VERSION(2,18,0)
+    gtk_widget_set_has_window(p->pwid,FALSE);
+#else
     GTK_WIDGET_SET_FLAGS(p->pwid, GTK_NO_WINDOW);
+#endif
     gtk_widget_set_name(p->pwid, "tray");
     gtk_container_set_border_width(GTK_CONTAINER(p->pwid), 1);
 
@@ -716,3 +775,5 @@ PluginClass tray_plugin_class = {
     panel_configuration_changed : tray_panel_configuration_changed
 
 };
+
+/* vim: set sw=4 sts=4 et : */
